# Generated by Django 5.2.4 on 2025-08-05 06:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("codenerix_payments", "0016_alter_paymentrequest_order_ref_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="paymentrequest",
            name="locator",
            field=models.CharField(
                max_length=64,
                unique=True,
                verbose_name="Locator",
            ),
        ),
    ]
