# Generated by Django 5.2.1 on 2025-05-22 06:14

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix_payments", "0014_alter_paymentrequest_order_ref"),
    ]

    operations = [
        migrations.AlterField(
            model_name="paymentrequest",
            name="order",
            field=models.PositiveIntegerField(
                validators=[
                    django.core.validators.MaxValueValidator(78364164096),
                ],
                verbose_name="Order Number",
            ),
        ),
    ]
