# Generated by Django 1.11 on 2020-04-29 11:31

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix_payments", "0006_auto_20190827_0759"),
    ]

    operations = [
        migrations.AlterField(
            model_name="paymentrequest",
            name="reverse",
            field=models.CharField(max_length=512, verbose_name="Reverse"),
        ),
    ]
