# Generated by Django 1.11 on 2019-08-06 06:08

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix_payments", "0002_auto_20190109_0752"),
    ]

    operations = [
        migrations.AlterField(
            model_name="paymentrequest",
            name="total",
            field=models.DecimalField(
                decimal_places=2,
                max_digits=10,
                verbose_name="Total",
            ),
        ),
    ]
