# Generated by Django 5.2.4 on 2025-07-07 13:59

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix_payments", "0015_alter_paymentrequest_order"),
    ]

    operations = [
        migrations.AlterField(
            model_name="paymentrequest",
            name="order_ref",
            field=models.CharField(
                max_length=8,
                verbose_name="Order Reference",
            ),
        ),
        migrations.AlterField(
            model_name="paymentrequest",
            name="reverse",
            field=models.CharField(
                default="autorender",
                max_length=64,
                verbose_name="Reverse",
            ),
        ),
    ]
