{% load i18n %}
<h1>{% trans "PAYMENT CONFIRMATION" %}</h1>
<table>
    <tr><td><u>{% trans "Action" %}:</u></td><td>{{action}}</td></tr>
    <tr><td><u>{% trans "Error" %}:</u></td><td>{{error}}</td></tr>
    {% if errortxt %}
        <tr><td><u>{% trans "Error explained" %}:</u></td><td>{{errortxt}}</td></tr>
    {% endif %}
    <tr><td><u>{% trans "Requested" %}:</u></td><td>{{request.request_date}}</td></tr>
    {% if confirmation %}
        <tr><td><u>{% trans "Answered" %}:</u></td><td>{{confirmation.answer_date}}</td></tr>
    {% endif %}
    <tr>
        <td>
            <u>{% trans "Status" %}:</u>
        </td>
        <td>
            <b>
            {% if confirmation %}
                <font color='green'>{% trans "CONFIRMED" %}</font>
            {% else %}
                <font color='red'>{% trans "NOT CONFIRMED" %}</font>
            {% endif %}
            </b>
        </td>
    </tr>
</table>
