# Generated by Django 1.11 on 2019-08-27 05:59

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix_payments", "0005_paymentrequest_feedback"),
    ]

    operations = [
        migrations.AddField(
            model_name="paymentanswer",
            name="ip",
            field=models.GenericIPAddressField(
                default="127.127.127.127",
                editable=False,
                verbose_name="IP",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="paymentconfirmation",
            name="ip",
            field=models.GenericIPAddressField(
                default="127.127.127.127",
                editable=False,
                verbose_name="IP",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="paymentrequest",
            name="ip",
            field=models.GenericIPAddressField(
                default="127.127.127.127",
                editable=False,
                verbose_name="IP",
            ),
            preserve_default=False,
        ),
    ]
