# Generated by Django 1.11 on 2019-08-14 05:42

import jsonfield.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix_payments", "0004_paymentrequest_user"),
    ]

    operations = [
        migrations.AddField(
            model_name="paymentrequest",
            name="feedback",
            field=jsonfield.fields.JSONField(
                blank=True,
                null=True,
                verbose_name="Feedback",
            ),
        ),
    ]
