# Generated by Django 4.1 on 2022-08-18 04:40

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix_payments", "0010_auto_20200706_1032"),
    ]

    operations = [
        migrations.AlterField(
            model_name="currency",
            name="price",
            field=models.DecimalField(
                decimal_places=2,
                max_digits=10,
                verbose_name="Price",
            ),
        ),
        migrations.AlterField(
            model_name="paymentrequest",
            name="reverse",
            field=models.CharField(max_length=64, verbose_name="Reverse"),
        ),
        migrations.AlterField(
            model_name="paymentrequest",
            name="total",
            field=models.DecimalField(
                decimal_places=2,
                max_digits=10,
                verbose_name="Total",
            ),
        ),
    ]
