<td align='center'>{{row.user|codenerix}}</td>
<td align='center'><span codenerix-html-compile='row.is_paid|codenerix'></span></td>
<td align='center' title="{{row.locator}}"><span codenerix-html-compile='row.locator|codenerix:"shorttext:5:page:2"'></span></td>
<td align='center' title="{{row.feedback}}"><span codenerix-html-compile='row.feedback|codenerix:"bool"'></span></td>
<td>{{row.order|codenerix}}</td>
<td>{{row.order_ref|codenerix}}</td>
<td>{{row.request_date|codenerix}}</td>
<td>{{row.answer_date|codenerix}}</td>
<td>{{row.platform|codenerix}}</td>
<td>{{row.protocol|codenerix}}</td>
<td>{{row.total|codenerix}}</td>
<td>{{row.currency|codenerix}}</td>
<td>
    <b>
        <span ng-if='row.cancelled' class='text-danger'>{{data.meta.gentranslate.yes}}</span>
        <span ng-if='!row.cancelled' class='text-success'>{{data.meta.gentranslate.no}}</span>
    </b>
</td>
<td>
    <b>
        <span ng-if='row.error' class='text-danger'>{{data.meta.gentranslate.yes}}</span>
        <span ng-if='!row.error' class='text-success'>{{data.meta.gentranslate.no}}</span>
    </b>
</td>
<td>{{row.notes|codenerix}}</td>
<td>{{row.ip|codenerix}}</td>
<td ng-click="$event.stopPropagation();" ng-controller="codenerixPaymentRequestCtrl">

    <span ng-if='row.is_paid || row.cancelled'>
        <span ng-if='row.is_paid && (!row.cancelled)'>
            <button class='btn btn-success btn-xs'>{{data.meta.gentranslate.paid}}</button>
        </span>
        <span ng-if='!row.is_paid && row.cancelled'>
            <button class='btn btn-secondary btn-xs'>{{data.meta.gentranslate.cancelled}}</button>
        </span>
        <span ng-if='row.is_paid && row.cancelled'>
            <button class='btn btn-danger btn-xs'>{{data.meta.gentranslate.error}}</button>
        </span>
    </span>
    <span ng-if='!row.is_paid && (!row.cancelled)'>
        <span ng-if='!row.get_approval_list.error'>
            <button ng-click="submit_approval(row.pk, row.get_approval_list)" ng-disabled='row.is_paid' class='btn btn-info btn-xs'>{{data.meta.gentranslate.pay}}</button>
            <div style='display:none' id='PaymentInternalFormHTML{{row.pk}}'></div>
        </span>
        <span ng-if='row.get_approval_list.error' class='text-danger'>{{row.get_approval_list.error}}</span>
        <button ng-click="submit_cancel(row.locator, data.meta.context.cancelurl)" class='btn btn-danger btn-xs'>{{data.meta.gentranslate.cancel}}</button>
    </span>
</td>
