# Generated by Django 5.2.1 on 2025-05-22 06:13

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix_payments", "0013_alter_paymentrequest_feedback_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="paymentrequest",
            name="order_ref",
            field=models.CharField(
                max_length=8,
                verbose_name="Order Reference",
            ),
        ),
    ]
