<td>{{row.payment__locator|codenerix}}</td>
<td>{{row.payment__order|codenerix}}</td>
<td>{{row.payment__order_ref|codenerix}}</td>
<td>{{row.created|codenerix}}</td>
<td>{{row.action|codenerix}}</td>
<td>{{row.payment__total|codenerix}}</td>
<td>{{row.payment__currency|codenerix}}</td>
<td>
    <b>
        <span ng-if='row.error=="True"' class='text-danger'>{{data.meta.gentranslate.yes}}</span>
        <span ng-if='row.error=="False"' class='text-success'>{{data.meta.gentranslate.no}}</span>
    </b>
</td>
<td>{{row.payment__ref|codenerix}}</td>
<td>{{row.ref|codenerix}}</td>
<td>{{row.ip|codenerix}}</td>
