# Generated by Django 5.2.1 on 2025-05-16 08:05

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "codenerix_payments",
            "0011_alter_currency_price_alter_paymentrequest_reverse_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="currency",
            name="price",
            field=models.DecimalField(
                decimal_places=4,
                max_digits=10,
                verbose_name="Price",
            ),
        ),
        migrations.AlterField(
            model_name="paymentrequest",
            name="total",
            field=models.DecimalField(
                decimal_places=4,
                max_digits=10,
                verbose_name="Total",
            ),
        ),
    ]
